@@ -6,14 +6,17 @@ from __future__ import absolute_import
 from builtins import zip
 # Simple edit of gwiki.py
 
-import re, os, sys, subprocess
-from .common import default_movie, plain_exercise, insert_code_and_tex
+import os, sys, subprocess
+import regex as re
+from .common import default_movie, plain_exercise, insert_code_blocks, insert_tex_blocks
 from .plaintext import plain_quiz
 from .misc import _abort, errwarn
 
+
 def cwiki_code(filestr, code_blocks, code_block_types,
                tex_blocks, format):
-    filestr = insert_code_and_tex(filestr, code_blocks, tex_blocks, format)
+    filestr = insert_code_blocks(filestr, code_blocks, format, complete_doc=True, remove_hid=True)
+    filestr = insert_tex_blocks(filestr, tex_blocks, format, complete_doc=True)
     c = re.compile(r'^!bc(.*?)\n', re.MULTILINE)
     filestr = c.sub(r'{{{\n', filestr)
     filestr = re.sub(r'!ec\n', r'}}}\n', filestr)
@@ -22,8 +25,23 @@ def cwiki_code(filestr, code_blocks, code_block_types,
     filestr = re.sub(r'!et\n', r'}}}\n', filestr)
     return filestr
 
+
 def cwiki_figure(m):
-    filename = m.group('filename')
+    """Format figures to the cwiki format
+
+    Return cwiki code to embed a figure in the sphinx .cwiki output. The syntax is
+    `FIGURE:[filename[, options][, sidecap=BOOL][, frac=NUM]] [caption]`.
+    Keywords: `sidecap` (default is False), `frac` (default is ),
+    :param _regex.Match m: regex match object
+    :return: cwiki code
+    :rtype: str
+    """
+    filename = m.group('filename').strip()
+    caption = m.group('caption').strip().strip('"').strip("'")
+    # inline options not implemented?
+    #opts = m.group('options').strip()
+    #info = dict()
+
     link = filename if filename.startswith('http') else None
     if not link and not os.path.isfile(filename):
         raise IOError('no figure file %s' % filename)
@@ -45,15 +63,17 @@ def cwiki_figure(m):
                 errwarn('Convert %s to PNG format manually' % filename)
                 _abort()
             filename = root + '.png'
-    caption = m.group('caption')
+
     # keep label if it's there:
     caption = re.sub(r'label\{(.+?)\}', '(\g<1>)', caption)
 
     result = r"""{{%s|%s}}""" % (filename, caption)
     return result
 
+
 from .common import table_analysis
 
+
 def cwiki_table(table):
     """Native Creole wiki table."""
     # add 2 chars for column width since we add boldface _..._
@@ -81,9 +101,8 @@ def cwiki_table(table):
     s += '\n\n'
     return s
 
-def cwiki_author(authors_and_institutions, auth2index,
-                 inst2index, index2inst, auth2email):
 
+def cwiki_author(authors_and_institutions, auth2index, inst2index, index2inst, auth2email):
     authors = []
     for author, i, email in authors_and_institutions:
         if email is None:
@@ -107,8 +126,10 @@ def cwiki_author(authors_and_institutions, auth2index,
     # we skip institutions in Creole wiki
     return text
 
+
 from .gwiki import wiki_ref_and_label_common
 
+
 def cwiki_ref_and_label(section_label2title, format, filestr):
     return wiki_ref_and_label_common(section_label2title, format, filestr)
 
