@@ -1,18 +1,22 @@
 from __future__ import absolute_import
 from builtins import str
 from builtins import range
-import re, sys, shutil, os
+import hashlib
+import sys, shutil, os
+import regex as re
+import shlex
+import hashlib
 from .common import default_movie, plain_exercise, table_analysis, indent_lines, \
     bibliography, fix_ref_section_chapter, cite_with_multiple_args2multiple_cites, \
-    _CODE_BLOCK, _MATH_BLOCK, DEFAULT_ARGLIST
+    _CODE_BLOCK, _MATH_BLOCK, DEFAULT_ARGLIST, envir_delimiter_lines
 from .doconce import INLINE_TAGS_SUBST, INLINE_TAGS
 from .latex import fix_latex_command_regex
-from .html import html_movie, html_table
+from .html import html_movie, html_table, html_toc
 from .pandoc import pandoc_ref_and_label, pandoc_index_bib, pandoc_quote, \
      language2pandoc, pandoc_quiz
-from .misc import option, errwarn, _abort
+from .misc import _doconce_header, _doconce_command, option, errwarn, _abort
 from doconce import globals
-from . import jupyter_execution
+from .jupyter_execution import JupyterKernelClient, get_code_block_args, get_execute_show, check_errors_in_code_output
 
 try:
     from nbformat.v4 import new_code_cell, new_markdown_cell, new_notebook, new_output
@@ -82,7 +86,7 @@ def ipynb_author(authors_and_institutions, auth2index,
             s += ' at ' + ' & '.join(i)
         s += ' -->\n'
         # Add extra line between heading and first author
-        s+= '<!-- Author: -->  \n_%s_' % (author)  
+        s+= '<!-- Author: -->  \n_%s_' % (author)
         '''
         # Write the authors
         s += '_%s_' % (author)
@@ -103,13 +107,33 @@ def ipynb_table(table):
     return text
 
 def ipynb_figure(m):
+    """Format figures for the ipynb format
+
+    Return json code to embed a figure in ipynb output. The syntax is
+    `FIGURE:[filename[, options][, sidecap=BOOL][, frac=NUM]] [caption]`.
+    Keywords: `sidecap` (default is False), `frac` (default is ),
+    Options: `--html_figure_hrule`, --html_figure_caption`, `--html_responsive_figure_width`
+    :param _regex.Match m: regex match object
+    :return: json code
+    :rtype: str
+    """
+    filename = m.group('filename').strip()
+    caption = m.group('caption').strip().strip('"').strip("'")
+    opts = m.group('options').strip()
+    info = dict()
+
+    # Process any inline figure opts
+    if opts:
+        info = shlex.split(opts)
+        info = dict(s.strip(',').split('=') for s in info)
+        # String of options
+        opts = ' '.join(['%s="%s"' % (opt, value)
+                         for opt, value in info.items()
+                         if opt not in ['frac', 'sidecap']])
+
     # m.group() must be called before m.group('name')
     text = '<!-- dom:%s -->\n<!-- begin figure -->\n' % m.group()
 
-    caption = m.group('caption').strip()
-    filename = m.group('filename')
-    opts = m.group('options').strip()
-
     # Get some global vars
     global figure_files
     if not filename.startswith('http'):
@@ -127,13 +151,6 @@ def ipynb_figure(m):
     if label is not None:
         text += '<div id="%s"></div>\n' % label
 
-    # Process any inline figure opts
-    if opts:
-        info = [s.split('=') for s in opts.split()]
-        opts = ' ' .join(['%s=%s' % (opt, value)
-                          for opt, value in info
-                          if opt not in ['frac', 'sidecap']])
-
     display_method = option('ipynb_figure=', 'imgtag')
     if display_method == 'md':
         # Markdown image syntax for embedded image in text
@@ -149,6 +166,8 @@ def ipynb_figure(m):
                                            opts=opts,
                                            caption=caption,
                                            figure_number=figure_number)
+        if not caption:
+            text = text.replace('Figure ' + str(figure_number) + ': ', 'Figure ' + str(figure_number))
     elif display_method == 'Image':
         # Image object
         # NOTE: This code will normally not work because it inserts a verbatim
@@ -298,6 +317,7 @@ def ipynb_code(filestr, code_blocks, code_block_types,
                      flags=re.DOTALL|re.MULTILINE)
     #filestr = re.sub('^<!-- !split -->\n', '', filestr, flags=re.MULTILINE)
 
+    # NB: some of the following environments might have already been processed
     envirs = globals.doconce_envirs[8:-2]
     for envir in envirs:
         pattern = r'^!b%s(.*?)\n(.+?)\s*^!e%s' % (envir, envir)
@@ -385,7 +405,8 @@ def ipynb_code(filestr, code_blocks, code_block_types,
     else:
         split_pyshell = True
 
-    ipynb_code_tp = [None]*len(code_blocks)
+    ipynb_code_type = [None]*len(code_blocks)
+    ipynb_code_postfix_err = [''] * len(code_blocks)
     for i in range(len(code_blocks)):
         # Check if continuation lines are in the code block, because
         # doconce.py inserts a blank after the backslash
@@ -400,17 +421,19 @@ def ipynb_code(filestr, code_blocks, code_block_types,
             code_blocks[i] = '%matplotlib inline\n\n' + code_blocks[i]
             mpl_inline = True
 
-        tp = code_block_types[i]
-        if tp.endswith('-t'):
+        code_block_type = code_block_types[i]
+        LANG, codetype, postfix, postfix_err = get_code_block_args('!bc ' + code_block_type)
+        ipynb_code_postfix_err[i] = postfix_err
+        if postfix == '-t':
             # Standard Markdown code with pandoc/github extension
-            language = tp[:-2]
+            language = code_block_type[:-2]
             language_spec = language2pandoc.get(language, '')
             #code_blocks[i] = '\n' + indent_lines(code_blocks[i], format) + '\n'
             code_blocks[i] = "```%s\n" % language_spec + \
                              indent_lines(code_blocks[i].strip(), format) + \
                              "```"
-            ipynb_code_tp[i] = 'markdown'
-        elif tp.startswith('pyshell') or tp.startswith('ipy'):
+            ipynb_code_type[i] = 'markdown'
+        elif LANG in ['pyshell', 'ipy']:
             lines = code_blocks[i].splitlines()
             last_cell_end = -1
             if split_pyshell:
@@ -430,7 +453,7 @@ def ipynb_code(filestr, code_blocks, code_block_types,
                             '\n'.join(lines[last_cell_end+1:j+1]))
                         last_cell_end = j
                 code_blocks[i] = new_code_blocks
-                ipynb_code_tp[i] = 'cell'
+                ipynb_code_type[i] = 'cell'
             else:
                 # Remove prompt and output lines; leave code executable in cell
                 for j in range(len(lines)):
@@ -445,8 +468,8 @@ def ipynb_code(filestr, code_blocks, code_block_types,
                 for j in range(lines.count('')):
                     lines.remove('')
                 code_blocks[i] = '\n'.join(lines)
-                ipynb_code_tp[i] = 'cell'
-        elif tp.startswith('sys'):
+                ipynb_code_type[i] = 'cell'
+        elif LANG == 'sys':
             # Do we find execution of python file? If so, copy the file
             # to separate subdir and make a run file command in a cell.
             # Otherwise, it is just a plain verbatim Markdown block.
@@ -459,31 +482,25 @@ def ipynb_code(filestr, code_blocks, code_block_types,
                     name = m.group(2).strip()
                     if os.path.isfile(name):
                         src_paths.add(os.path.dirname(name))
-                        lines[j] = '%%run "%s"' % fullpath
+                        lines[j] = '%%run "%s"' % name
                 else:
                     found_unix_lines = True
             src_paths = list(src_paths)
             if src_paths and not found_unix_lines:
                 # This is a sys block with run commands only
                 code_blocks[i] = '\n'.join(lines)
-                ipynb_code_tp[i] = 'cell'
+                ipynb_code_type[i] = 'cell'
             else:
                 # Standard Markdown code
                 code_blocks[i] = '\n'.join(lines)
                 code_blocks[i] = indent_lines(code_blocks[i], format)
-                ipynb_code_tp[i] = 'markdown'
-        elif tp.endswith("-e"):
-            ipynb_code_tp[i] = 'cell_execute_hidden'
-        elif tp.endswith('hid'):
-            ipynb_code_tp[i] = 'cell_hidden'
-        elif tp.endswith('out'):
-            ipynb_code_tp[i] = 'cell_output'
-        elif tp.startswith('py'):
-            ipynb_code_tp[i] = 'cell'
+                ipynb_code_type[i] = 'markdown'
+        elif LANG.startswith('py'):
+            ipynb_code_type[i] = 'cell'
         else:
             # Should support other languages as well, but not for now
             code_blocks[i] = indent_lines(code_blocks[i], format)
-            ipynb_code_tp[i] = 'markdown'
+            ipynb_code_type[i] = 'markdown'
 
     # figure_files and movie_files are global variables and contain
     # all figures and movies referred to
@@ -501,55 +518,93 @@ def ipynb_code(filestr, code_blocks, code_block_types,
     elif os.path.isfile(ipynb_tarfile):
         os.remove(ipynb_tarfile)
 
-
     # Parse document into markdown text, code blocks, and tex blocks.
     # Store in nested list notebook_blocks.
+    # Also remove some commented markers
     notebook_blocks = [[]]
-    authors = ''
+    marker_subex_begin = INLINE_TAGS_SUBST[format]['comment'] % envir_delimiter_lines['subex'][0]
+    markers2rm = []
+    markers2rm.append(INLINE_TAGS_SUBST[format]['comment'] % envir_delimiter_lines['subex'][1])
+    markers2rm.append(INLINE_TAGS_SUBST[format]['comment'] % envir_delimiter_lines['exercise'][0])
+    markers2rm.append(INLINE_TAGS_SUBST[format]['comment'] % envir_delimiter_lines['exercise'][1])
+    notebook_block_envir = code_block_types
     for line in filestr.splitlines():
-        if line.startswith('authors = [new_author(name='):  # old author method
-            authors = line[10:]
-        elif _CODE_BLOCK in line:
-            code_block_tp = line.split()[-1]
-            if code_block_tp in ('pyhid',) or not code_block_tp.endswith('hid'):
-                notebook_blocks[-1] = '\n'.join(notebook_blocks[-1]).strip()
-                notebook_blocks.append(line)
-            # else: hidden block to be dropped (may include more languages
-            # with time in the above tuple)
-        elif _MATH_BLOCK in line:
-            notebook_blocks[-1] = '\n'.join(notebook_blocks[-1]).strip()
-            notebook_blocks.append(line)
-        elif 0 and line.startswith('# '):
-            if not isinstance(notebook_blocks[-1], list):
-                notebook_blocks.append([])
-            notebook_blocks[-1] = '\n'.join(notebook_blocks[-1]).strip()
-        elif '!split' in line:
-            if not isinstance(notebook_blocks[-1], list):
-                notebook_blocks.append([])
-            notebook_blocks[-1] = '\n'.join(notebook_blocks[-1]).strip()
-        else:
-            if not isinstance(notebook_blocks[-1], list):
-                notebook_blocks.append([])
+        if line == '':
+            notebook_blocks[-1].append('\n')
+        elif line.startswith('#'):
+            # Each chapter, sub/sub/section in its own cell
+            notebook_blocks.append([])
             notebook_blocks[-1].append(line)
-    if isinstance(notebook_blocks[-1], list):
-        notebook_blocks[-1] = '\n'.join(notebook_blocks[-1]).strip()
+        elif line in marker_subex_begin:
+            # Each subex in its own cell
+            notebook_blocks.append([])
+        elif line in markers2rm:
+            # Remove the markers for begin/end of exercises and subexercises
+            notebook_blocks[-1].append('\n')
+        elif _CODE_BLOCK in line:
+            # Do not write block with `-hid` postfix except `py-hid`, which is a collapsed block
+            LANG, codetype, postfix, postfix_err = get_code_block_args('!bc ' + line.split()[-1])
+            if not postfix == '-hid' or (LANG == 'py' and postfix == '-hid'):
+                notebook_blocks.append([line])
+                notebook_blocks.append([])
+            else:
+                # mark block type for removal
+                pattern = r'(\d+) +%s'
+                m = re.match(pattern % _CODE_BLOCK, line)
+                idx = int(m.group(1))
+                notebook_block_envir[idx] = 'remove'
+        elif _MATH_BLOCK in line:
+            notebook_blocks.append([line])
+            notebook_blocks.append([])
+        elif '!split' in line:
+            # !split starts a new exercise (therefore it cannot be placed between subex, sol, ans, etc)
+            notebook_blocks.append([])
+        elif envir_delimiter_lines['subex'][0] in line:
+            notebook_blocks[-1] = '\n'.join(notebook_blocks[-1]).strip()
+            notebook_blocks.append([])
+        else:
+            notebook_blocks[-1].append(line)
+    # Join text in each element of notebook_blocks removing multiple newlines
+    for i, block in enumerate(notebook_blocks):
+        block = '\n'.join(block).strip()
+        notebook_blocks[i] = re.sub(r'\n\n+','\n\n', block)
     # Remove empty blocks
     notebook_blocks = [b for b in notebook_blocks if b != '']
 
     # Add block type info
     pattern = r'(\d+) +%s'
+    notebook_block_types = [''] * len(notebook_blocks)
+    # Description of relevant variables
+    # notebook_blocks :         ['Some text', 'import sys', 'print(sys.version)', 'x <- 1:6']
+    # notebook_block_types :    ['text', 'code', 'code', 'code']
+    # notebook_block_envir :    ['py-hid', 'py-err']
     for i in range(len(notebook_blocks)):
         if re.match(pattern % _CODE_BLOCK, notebook_blocks[i]):
             m = re.match(pattern % _CODE_BLOCK, notebook_blocks[i])
             idx = int(m.group(1))
-            if ipynb_code_tp[idx].startswith('cell'):
-                notebook_blocks[i] = [ipynb_code_tp[idx], notebook_blocks[i]]
+            if ipynb_code_type[idx] == 'cell':
+                notebook_block_types[i] = ipynb_code_type[idx]
             else:
-                notebook_blocks[i] = ['text', notebook_blocks[i]]
+                notebook_block_types[i] = 'text'
+                # mark block type for removal
+                notebook_block_envir[idx] = 'remove'
         elif re.match(pattern % _MATH_BLOCK, notebook_blocks[i]):
-            notebook_blocks[i] = ['math', notebook_blocks[i]]
+            notebook_block_types[i] = 'math'
         else:
-            notebook_blocks[i] = ['text', notebook_blocks[i]]
+            notebook_block_types[i] = 'text'
+
+    # Remove some block environments: text (`-t` postfix) and unsupported/hidden formats (e.g. `f-hid`)
+    notebook_block_envir = list(filter(lambda env: env != 'remove', notebook_block_envir))
+
+    # Sanity check - issue 193
+    if len(notebook_blocks) != len(notebook_block_types):
+        errwarn('*** error: internal error')
+        errwarn('    each code block should have a type')
+        _abort()
+    elif len(notebook_block_envir) != len([t for t in notebook_block_types if t == 'cell']):
+        errwarn('*** error: internal error')
+        errwarn('    each code block should have a code environment')
+        _abort()
 
     # Go through tex_blocks and wrap math blocks in $$
     # (doconce.py runs align2equations so there are no align/align*
@@ -606,64 +661,122 @@ def ipynb_code(filestr, code_blocks, code_block_types,
     # blocks is now a list of text chunks in markdown and math/code line
     # instructions. Insert code and tex blocks
     for i in range(len(notebook_blocks)):
-        if _CODE_BLOCK in notebook_blocks[i][1] or _MATH_BLOCK in notebook_blocks[i][1]:
-            words = notebook_blocks[i][1].split()
+        if _CODE_BLOCK in notebook_blocks[i] or _MATH_BLOCK in notebook_blocks[i]:
+            words = notebook_blocks[i].split()
             # start of notebook_blocks[i]: number block-indicator code-type
             n = int(words[0])
-            if _CODE_BLOCK in notebook_blocks[i][1]:
-                notebook_blocks[i][1] = code_blocks[n]  # can be list!
-            if _MATH_BLOCK in notebook_blocks[i][1]:
-                notebook_blocks[i][1] = tex_blocks[n]
+            if _CODE_BLOCK in notebook_blocks[i]:
+                notebook_blocks[i] = code_blocks[n]  # can be list!
+            if _MATH_BLOCK in notebook_blocks[i]:
+                notebook_blocks[i] = tex_blocks[n]
+
+    # Prepend the doconce header and command to the first text cell
+    intro = _doconce_header + '\n'
+    intro += _doconce_command % ('ipynb', globals.filename, ' '.join(sys.argv[1:]))
+    intro = INLINE_TAGS_SUBST[format]['comment'] % intro + '\n\n'
+    ind = next((i for i, type in enumerate(notebook_block_types) if type == 'text'), -1)
+    if ind > -1:
+        notebook_blocks[ind] = intro + notebook_blocks[ind]
 
     global new_code_cell, new_markdown_cell, new_notebook, new_output
     cells = []              # ipynb cells
-    mdstr = []              # plain md format of the notebook
+    #mdstr = []             # plain md format of the notebook
     execution_count = 1
     kernel_client = None    # Placeholder for a JupyterKernelClient instance
     if option("execute"):
-        kernel_client = jupyter_execution.JupyterKernelClient()
+        kernel_client = JupyterKernelClient(syntax='python')
     editable_md = True      # Metadata for md text
     if option('ipynb_non_editable_text'):
         editable_md = False
 
-    for block_tp, block in notebook_blocks:
+    j = 0
+    for i, block in enumerate(notebook_blocks):
+        block_tp = notebook_block_types[i]
         if (block_tp == 'text' or block_tp == 'math') and block != '' and block != '<!--  -->':
             block = re.sub(r"caption\{(.*)\}", r"*Figure: \1*", block)
             cells.append(new_markdown_cell(source=block, metadata=dict(editable=editable_md)))
-            mdstr.append(('markdown', block))
-        elif block_tp == 'cell_output' and block != '' and not option("ignore_output"):
-            # Process cells with block type cell_output (*out)
-            block = block.rstrip()
-            outputs = [
-                new_output(
-                    output_type="execute_result",
-                    data={
-                        "text/plain": [
-                            block
-                          ]
-                    },
-                    execution_count=execution_count-1
-                )
-            ]
-            previous_cell = cells[-1]
-            if previous_cell.cell_type == "code":
-                previous_cell.outputs = outputs
+            #mdstr.append(('markdown', block))
+        elif block_tp == 'cell':
+            # TODO: now I need to handle -hid also in 'markdown' formats. The logic on postfixes should actually be
+            # separated from LANG
+            LANG, codetype, postfix, postfix_err = get_code_block_args('!bc ' + notebook_block_envir[j])
+            j += 1
+            execute, show = get_execute_show((LANG, codetype, postfix))
+            current_code_envir = LANG + codetype + postfix + postfix_err or 'dat'
+            if show == 'output' and block != '' and not option('ignore_output'):
+                # Process cells with output block type (`-out` postfix)
+                block = block.rstrip()
+                outputs = [
+                    new_output(
+                        output_type="execute_result",
+                        data={
+                            "text/plain": [
+                                block
+                              ]
+                        },
+                        execution_count=execution_count-1
+                    )
+                ]
+                previous_cell = cells[-1]
+                if previous_cell.cell_type == "code":
+                    previous_cell.outputs = outputs
+                else:
+                    print("WARNING: DocOnce ipynb got code output,",
+                          "but previous was not code.")
+                    cell = new_code_cell(
+                        source="#",
+                        outputs=outputs,
+                        execution_count=execution_count,
+                        metadata=dict(collapsed=False)
+                    )
+                    cells.append(cell)
+                    #mdstr.append(('codecell', block))
             else:
-                print("WARNING: DocOnce ipynb got code output,",
-                      "but previous was not code.")
-                cell = new_code_cell(
-                    source="#",
-                    outputs=outputs,
-                    execution_count=execution_count,
-                    metadata=dict(collapsed=False)
-                )
-                cells.append(cell)
-                mdstr.append(('codecell', block))
-        elif block_tp.startswith('cell'):
-            # Process cells with block type cell (py*), cell_execute_hidden (*-e), cell_hidden (*hid)
-            execution_count, md_out = process_ipynb_code_block(kernel_client, block, block_tp, cells, execution_count)
-            mdstr.extend(md_out)
+                # Process cells with block type cell (py*), (*-e), (*-hid)
+                cells_output, execution_count, error = execute_code_block(block,
+                                                                  current_code_envir,
+                                                                  kernel_client,
+                                                                  execution_count)
+                # Warn and abort on code errors
+                if error != '':
+                    if option('execute=') == 'abort':
+                        errwarn('*** error: Error in code block:')
+                    else:
+                        errwarn('*** Warning: found error in code block:')
+                    errwarn('    %s' % error)
+                    errwarn('***')
+                    if option('execute=') == 'abort' and not postfix_err:
+                        _abort()
+                # Add the cell, except when the `-e` postfix is used. `-hid` is just a collapsed cell
+                if postfix != '-e':
+                    #mdstr.extend(md_out)
+                    for cell in cells_output:
+                        if cell:
+                            cells.append(cell)
+    """
+    By default, each cell gets a random unique ID
+    (currently, `uuid.uuid4().hex[:8]` in nbformat/corpus/words.py)
+    To ensure cells do not change ID every time they are compiled,
+    replace this ID with a hash of the 'source' field.
+    If needed, add a running number to duplicates to ensure uniqueness.
+    """
+    # Replace the random id with a reproducible hash of the content (issue #213)
+    # nbformat 5.1.3 creates random cell ID with `uuid.uuid4().hex[:8]`
+    hashed_ids = {} # dict of created hashes
+    for i in range(len(cells)):
+        if 'id' in cells[i].keys():
+            cell_content = cells[i]['source']
+            hashed_id = hashlib.sha224(cell_content.encode()).hexdigest()[:8]
+            cells[i]['id'] = hashed_id
+            # check for dupliatce IDs
+            if hashed_id in hashed_ids:
+                # append running number for each next one,
+                # use the current count for this hash to append _1, _2, ...
+                cells[i]['id'] = hashed_id + "_" + str(hashed_ids[hashed_id])
+            # add to dict with existing IDs
+            hashed_ids[hashed_id] = hashed_ids.get(hashed_id, 0) + 1
 
+    # Create the notebook in string format
     nb = new_notebook(cells=cells)
     filestr = writes(nb, version=4)
 
@@ -737,18 +850,34 @@ def ipynb_code(filestr, code_blocks, code_block_types,
                '}')
     '''
     if option("execute"):
-        jupyter_execution.stop(kernel_client)
+        kernel_client.stop()
 
     return filestr
 
 
-def process_ipynb_code_block(kernel_client, block, block_tp, cells, execution_count):
-    md_out = []
+def execute_code_block(block, current_code_envir, kernel_client, execution_count):
+    """Execute a code block and return the ipynb output
+
+    Execute a code block in a jupyter kernel. Return the ipynb output for the ipynb format
+    together with the execution count and any code error
+    :param str block: code block
+    :param str current_code_envir: code environment LANG + codetype + postfix + postfix_err
+    :param JupyterKernelClient kernel_client: instance of JupyterKernelClient
+    :param int execution_count: execution count in the rendered cell
+    :return: notebook cells, execution_count, error
+    :rtype List[NotebookNode], int, str
+    """
+    #md_out = []
+    cell = None
+    cells = []
+    error = ''
     editable = True
     collapsed = False
     if option('ipynb_non_editable_code'):
-        editable=False
-    if block_tp == 'cell_hidden':
+        editable = False
+    LANG, codetype, postfix, postfix_err = get_code_block_args('!bc ' + current_code_envir)
+    # The `-hid` postfix collapses the cell
+    if postfix == '-hid':
         collapsed = True
     # Start processing the block
     if not isinstance(block, list):
@@ -761,16 +890,27 @@ def process_ipynb_code_block(kernel_client, block, block_tp, cells, execution_co
             execution_count=execution_count,
             metadata=dict(editable=editable, collapsed=collapsed)
         )
-        if block_tp != 'cell_execute_hidden':
-            cells.append(cell)
-        if option("execute"):
-            outputs, execution_count_out = jupyter_execution.run_cell(kernel_client, blockline)
+        if option("execute") and postfix != "-noex":
+            outputs, execution_count_out = kernel_client.run_cell(blockline)
+            # Extract any error in code
+            error = check_errors_in_code_output(outputs, execution_count)
+            for output in outputs:
+                if output['output_type'] == 'error':
+                    traceback = ''
+                    ansi_escape = re.compile(r'\x1b[^m]*m')
+                    if "traceback" in output:
+                        traceback = "\n".join(output["traceback"])
+                        # Escape characters
+                        traceback = ansi_escape.sub("", traceback)
+                    error = traceback or 'block #%d' % execution_count
+                    break
             cell.outputs = outputs
             if execution_count_out:
                 cell["execution_count"] = execution_count_out
+        cells.append(cell)
         execution_count += 1
-        md_out.append(('codecell', blockline))
-    return execution_count, md_out
+        #md_out.append(('codecell', blockline))
+    return cells, execution_count, error #, md_out
 
 
 def ipynb_index_bib(filestr, index, citations, pubfile, pubdata):
@@ -799,12 +939,12 @@ def ipynb_index_bib(filestr, index, citations, pubfile, pubdata):
         os.chdir(this_dir)
 
         bibstyle = option('latex_bibstyle=', 'plain')
-        bibtext = fix_latex_command_regex(r(
+        bibtext = fix_latex_command_regex((
             "((*- extends 'latex_article.tplx' -*))"
             ''
             '((* block bibliography *))'
-            '\bibliographystyle{%s}'
-            '\bibliography{%s}'
+            r'\bibliographystyle{%s}'
+            r'\bibliography{%s}'
             '((* endblock bibliography *))'
         ) % (bibstyle, bibtexfile[:-4]), application='replacement')
         filestr = re.sub(r'^BIBFILE:.+$', bibtext, filestr,
@@ -846,6 +986,7 @@ def ipynb_index_bib_latex_plain(filestr, index, citations, pubfile, pubdata):
 
 
 def ipynb_ref_and_label(section_label2title, format, filestr):
+    # NB: figure numbering already done in `handle_figures` > `ipynb_figure`
     filestr = fix_ref_section_chapter(filestr, format)
 
     # Replace all references to sections. Pandoc needs a coding of
@@ -929,7 +1070,7 @@ def define(FILENAME_EXTENSION,
         'linkURL3v': r'[`\g<link>`](\g<url>)',
         'plainURL':  r'<\g<url>>',
         'colortext': r'<font color="\g<color>">\g<text></font>',
-        'title':     r'<!-- dom:TITLE: \g<subst> -->\n# \g<subst>',
+        'title':     r'# \g<subst>',
         'author':    ipynb_author,
         'date':      '\nDate: _\g<subst>_\n',
         'chapter':       lambda m: '# '   + m.group('subst'),  # seldom used in notebooks
@@ -974,7 +1115,7 @@ def define(FILENAME_EXTENSION,
     else:
         INDEX_BIB['ipynb'] = pandoc_index_bib
     EXERCISE['ipynb'] = plain_exercise
-    TOC['ipynb'] = lambda s, f: ''
+    TOC['ipynb'] = html_toc
     FIGURE_EXT['ipynb'] = {
         'search': ('.png', '.gif', '.jpg', '.jpeg', '.tif', '.tiff'),
         'convert': ('.png', '.gif', '.jpg')}
