@@ -3,130 +3,202 @@ from __future__ import division
 from __future__ import unicode_literals
 
 import os
-
-from jupyter_client import KernelManager
+import subprocess
+from jupyter_client import KernelManager, kernelspec
 from nbformat.v4 import output_from_msg
 from . import misc
 try:
     from queue import Empty  # Py 3
 except ImportError:
     from Queue import Empty # Py 2
-import re, base64, uuid
+import base64, uuid
+import regex as re
 from .misc import errwarn, _abort, option
-from .common import safe_join, process_code_envir_postfix
+from .globals import envir2syntax, syntax_executable
+from .common import safe_join, get_code_block_args
+from pygments.lexers import get_lexer_by_name
+from html import escape, unescape
+
 class JupyterKernelClient:
-    def __init__(self):
-        self.manager = KernelManager(kernel_name='python3')
+    """Create and start a Jupyter kernel in a programming language.
+
+    Use the jupyter-client API to create and start a jupyter kernel, which can be used to run code.
+    The input programming language is searched between the list of installed Jupyter kernels.
+    .. seealso::
+        - jupyter-client: https://jupyter-client.readthedocs.io/en/latest/index.html
+        - Jupyter kernels: https://github.com/jupyter/jupyter/wiki/Jupyter-kernels
+    .. note::
+        - Install the IJulia Jupyter kernel with `using Pkg; Pkg.add("IJulia");Pkg.build("IJulia")` in julia
+        - Uninstall IJulia: `Pkg.rm("IJulia")` in julia, then `jupyter kernelspec uninstall mykernel`
+        - Install the bash kernel: https://github.com/takluyver/bash_kernel
+        also make sure the bash_kernel package is installed: `pip install bash_kernel`
+        - Install the IR kernel for R: https://irkernel.github.io/installation/
+        after upgrading to R 3.4+: https://github.com/duckmayr/install-update-r-on-linux
+        - List the currently installed kernels: jupyter kernelspec list
+    """
+    def __init__(self, syntax):
+        """Initialize the class by creating and starting a Jupyter kernel
+
+        :param str syntax: programming language of the Jupyter kernel
+        """
+        kernel_name = self.find_kernel_name(syntax)
+        self.kernel_name = kernel_name
+        self.manager = KernelManager(kernel_name=kernel_name)
+        if not self.kernel_name:
+            return
         self.kernel = self.manager.start_kernel()
         self.client = self.manager.client()
         self.client.start_channels()
 
-def stop(kernel_client):
-    kernel_client.client.stop_channels()
-    kernel_client.manager.shutdown_kernel()
+    def is_alive(self):
+        """Check if the kernel process is running
 
+        :return: True or False
+        :rtype: bool
+        """
+        return self.manager.is_alive()
 
-def run_cell(kernel_client, source, timeout=120):
-    if misc.option('verbose-execute'):
-        print("Executing cell:\n{}\nExecution in\n{}\n".format(source, os.getcwd()))
+    @staticmethod
+    def find_kernel_name(syntax):
+        """Internal function to searched a Jupyter kernel between the list of
+        Jupyter kernels installed.
 
-    # Adapted from nbconvert.ExecutePreprocessor
-    # Copyright (c) IPython Development Team.
-    # Distributed under the terms of the Modified BSD License.
-    msg_id = kernel_client.client.execute(source)
-    # wait for finish, with timeout
-    while True:
-        try:
-            msg = kernel_client.client.shell_channel.get_msg(timeout=timeout)
-        except Empty:
-            print("Timeout waiting for execute reply", timeout)
-            print("Tried to run the following source:\n{}".format(source))
-            try:
-                exception = TimeoutError
-            except NameError:
-                exception = RuntimeError
-            raise exception("Cell execution timed out")
-
-        if msg['parent_header'].get('msg_id') == msg_id:
-            if misc.option('verbose-execute'):
-                print("Wrong message id")
-            break
+        :param str syntax: programming language of the Jupyter kernel
+        :return: kernel name
+        :rtype: str or None
+        """
+        # Get Kernel specifications
+        # Kernel names look like: ['python3', 'ir', 'julia-1.6', 'bash']
+        # Kernel languages look like: ['python', 'R', 'julia', 'bash']
+        # Kernel display names look like: ['Python 3', 'R', 'Julia 1.6.1', 'Bash']
+        kernelspecmanager = kernelspec.KernelSpecManager()
+        kernel_specs = kernelspecmanager.get_all_specs()
+        kernel_name = [spec for spec in kernel_specs]
+        languages = [kernel_specs[spec]['spec']['language'].lower() for spec in kernel_specs]
+        display_names = [kernel_specs[spec]['spec']['display_name'] for spec in kernel_specs]
+        if syntax in languages:
+            return kernel_name[languages.index(syntax)]
         else:
-            if misc.option('verbose-execute'):
-                print("Not our reply")
-            continue
+            errwarn('*** No Jupyter kernels found for %s. The kernels available are:' % syntax)
+            errwarn('    %s' % ','.join(kernel_name))
+            errwarn('    To install Jupyter kernels see https://github.com/jupyter/jupyter/wiki/Jupyter-kernels')
+            return ''
 
-    outs = []
-    execution_count = None
-
-    while True:
-        try:
-            # We've already waited for execute_reply, so all output
-            # should already be waiting. However, on slow networks, like
-            # in certain CI systems, waiting < 1 second might miss messages.
-            # So long as the kernel sends a status:idle message when it
-            # finishes, we won't actually have to wait this long, anyway.
-            msg = kernel_client.client.iopub_channel.get_msg(timeout=5)
-        except Empty:
-            if misc.option('verbose-execute'):
-                print("Timeout waiting for IOPub output")
-            break
-        if msg['parent_header'].get('msg_id') != msg_id:
-            if misc.option('verbose-execute'):
-                print("Not output from our execution")
-            continue
+    def run_cell(self, source, timeout=120):
+        """Execute code in kernel
 
+        :param str source: code to run
+        :param int timeout: timeout
+        :return: execution output, execution count
+        :rtype: Tuple[List[str], int]
+        """
+        outs = []
+        execution_count = None
+        if 'client' not in self.__dir__():
+            outs, execution_count
         if misc.option('verbose-execute'):
-            print(msg)
+            print("Executing cell:\n{}\nExecution in\n{}\n".format(source, os.getcwd()))
 
-        msg_type = msg['msg_type']
-        content = msg['content']
+        # Adapted from nbconvert.ExecutePreprocessor
+        # Copyright (c) IPython Development Team.
+        # Distributed under the terms of the Modified BSD License.
+        msg_id = self.client.execute(source)
+        # wait for finish, with timeout
+        while True:
+            try:
+                msg = self.client.get_shell_msg(timeout=timeout)
+            except Empty:
+                print("Timeout waiting for execute reply", timeout)
+                print("Tried to run the following source:\n{}".format(source))
+                try:
+                    exception = TimeoutError
+                except NameError:
+                    exception = RuntimeError
+                raise exception("Cell execution timed out")
 
-        # set the prompt number for the input and the output
-        if 'execution_count' in content:
-            execution_count = content['execution_count']
-            # cell['execution_count'] = content['execution_count']
-
-        if msg_type == 'status':
-            if content['execution_state'] == 'idle':
+            if msg['parent_header'].get('msg_id') == msg_id:
                 if misc.option('verbose-execute'):
-                    print("State is idle")
+                    print("Wrong message id")
                 break
             else:
                 if misc.option('verbose-execute'):
-                    print("Other status")
+                    print("Not our reply")
                 continue
-        elif msg_type == 'execute_input':
-            continue
-        elif msg_type == 'clear_output':
-            outs[:] = []
-            if misc.option('verbose-execute'):
-                print("Request to clear output")
-            continue
-        elif msg_type.startswith('comm'):
-            if misc.option('verbose-execute'):
-                print("Output start with 'comm'")
-            continue
 
-        display_id = None
-        if msg_type in {'execute_result', 'display_data', 'update_display_data'}:
-            display_id = msg['content'].get('transient', {}).get('display_id', None)
-            if msg_type == 'update_display_data':
+        while True:
+            try:
+                # We've already waited for execute_reply, so all output
+                # should already be waiting. However, on slow networks, like
+                # in certain CI systems, waiting < 1 second might miss messages.
+                # So long as the kernel sends a status:idle message when it
+                # finishes, we won't actually have to wait this long, anyway.
+                msg = self.client.get_iopub_msg(timeout=5)
+            except Empty:
                 if misc.option('verbose-execute'):
-                    print("Update_display_data doesn't get recorded")
+                    print("Timeout waiting for IOPub output")
+                break
+            if msg['parent_header'].get('msg_id') != msg_id:
+                if misc.option('verbose-execute'):
+                    print("Not output from our execution")
                 continue
 
-        try:
-            out = output_from_msg(msg)
-        except ValueError:
-            print("unhandled iopub msg: " + msg_type)
+            if misc.option('verbose-execute'):
+                print(msg)
 
-        outs.append(out)
+            msg_type = msg['msg_type']
+            content = msg['content']
 
-        if misc.option('verbose-execute'):
-            print("Cell execution result:\n{}\n".format(out))
+            # set the prompt number for the input and the output
+            if 'execution_count' in content:
+                execution_count = content['execution_count']
+                # cell['execution_count'] = content['execution_count']
 
-    return outs, execution_count
+            if msg_type == 'status':
+                if content['execution_state'] == 'idle':
+                    if misc.option('verbose-execute'):
+                        print("State is idle")
+                    break
+                else:
+                    if misc.option('verbose-execute'):
+                        print("Other status")
+                    continue
+            elif msg_type == 'execute_input':
+                continue
+            elif msg_type == 'clear_output':
+                outs[:] = []
+                if misc.option('verbose-execute'):
+                    print("Request to clear output")
+                continue
+            elif msg_type.startswith('comm'):
+                if misc.option('verbose-execute'):
+                    print("Output start with 'comm'")
+                continue
+
+            display_id = None
+            if msg_type in {'execute_result', 'display_data', 'update_display_data'}:
+                display_id = msg['content'].get('transient', {}).get('display_id', None)
+                if msg_type == 'update_display_data':
+                    if misc.option('verbose-execute'):
+                        print("Update_display_data doesn't get recorded")
+                    continue
+
+            try:
+                out = output_from_msg(msg)
+            except ValueError:
+                print("unhandled iopub msg: " + msg_type)
+
+            outs.append(out)
+
+            if misc.option('verbose-execute'):
+                print("Cell execution result:\n{}\n".format(out))
+
+        return outs, execution_count
+
+    def stop(self):
+        """Shutdown the kernel."""
+        if 'client' in self.__dir__():
+            self.client.stop_channels()
+            self.manager.shutdown_kernel()
 
 
 # Helper functions to format code blocks, currently to the html, latex, and pdflatex formats
@@ -161,6 +233,64 @@ latex_caption = "   \\captionof{{figure}}{{{caption}}}\n"
 latex_label = "   \\label{{{label}}}\n"
 
 
+def start_kernel(syntax, kernel_client):
+    """Start a Jupyter kernel for a specified programming language
+
+    Given a programming language `syntax`, start a Jupyter kernel, if not already started.
+    The Jupyter kernels are started using the JupyterKernelClient class and inserted
+    in the input kernels `kernel_client` dictionary
+    :param str syntax: the kernel's programming language
+    :param Dict[str -> JupyterKernelClient] kernel_client: dictionary of kernel names to
+    instances of (JupyterKernelClient)
+    :param Dict[str -> JupyterKernelClient] kernels: dictionary of JupyterKernelClient instances
+    """
+    if option("execute"):
+        if syntax in kernel_client:
+            # Already started
+            pass
+        else:
+            kernel_client[syntax] = JupyterKernelClient(syntax=syntax)
+            if syntax == 'python':
+                outputs, count = kernel_client[syntax].run_cell(
+                    "%matplotlib inline\n" +
+                    "from IPython.display import set_matplotlib_formats\n" +
+                    "set_matplotlib_formats('pdf', 'png')"
+                )
+
+
+def execute_subprocess(code, syntax, code_style='', format=None):
+    """WORK IN PROGRESS. Execute code using Python's subprocess
+
+    Execute code using Python's subprocess and return a formatted output
+    :param str code: code to execute
+    :param str syntax: programming language of the code
+    :param str code_style: optional typesetting of code blocks
+    :param str format: output formatting, one of ['html', 'latex', 'pdflatex']
+    :return: formatted_output or ''
+    :rtype: str
+    """
+    formatted_output = ''
+    execution_count = 0
+    error = ''
+    basename = 'tmp_code'
+    syntax2suffix = dict(zip(envir2syntax.values(), envir2syntax.keys()))
+    syntax2run = {'bash': 'bash', 'r': 'Rscript', 'julia': 'julia', 'java': 'javac'}
+    if syntax in ['python', 'bash', 'julia', 'r']:  # add 'console'?
+        # Save to <file>.<envir>
+        filename_code = basename + '.' + syntax2suffix[syntax]
+        f = open(filename_code, 'w')
+        f.write(code)
+        f.close()
+        # Execute the file
+        result = subprocess.run(syntax2run[syntax] + ' ' + filename_code, stdout=subprocess.PIPE, shell=True)
+        result.stdout = result.stdout.decode("utf-8")
+        begin, end = formatted_code_envir("bash", code_style, format)
+        formatted_output = "\n{}\n{}{}".format(begin, result.stdout, end)
+    else:
+        print(formatted_output)
+    return formatted_output, execution_count, error
+
+
 def process_code_blocks(filestr, code_style, format):
     """Process a filestr with code blocks
 
@@ -175,45 +305,19 @@ def process_code_blocks(filestr, code_style, format):
     lines = filestr.splitlines()
     current_code_envir = None
     current_code = ""
-    kernel_client = None
-    if option("execute"):
-        kernel_client = JupyterKernelClient()
-        # This enables PDF output as well as PNG for figures.
-        # We only use the PDF when available, but PNG should be added as fallback.
-        outputs, count = run_cell(
-            kernel_client,
-            "%matplotlib inline\n" +
-            "from IPython.display import set_matplotlib_formats\n" +
-            "set_matplotlib_formats('pdf', 'png')"
-        )
-
+    kernel_client = {}
     for i in range(len(lines)):
         if lines[i].startswith('!bc'):
-            words = lines[i].split()
-            if len(words) == 1:
-                current_code_envir = 'ccq'
-            else:
-                current_code_envir = words[1]
-            if current_code_envir is None:
-                # Should not happen since a !bc is encountered first and
-                # current_code_envir is then set above
-                # There should have been checks for this in doconce.py
-                errwarn('*** error: mismatch between !bc and !ec')
-                errwarn('\n'.join(lines[i - 3:i + 4]))
-                _abort()
+            LANG, codetype, postfix, postfix_err = get_code_block_args(lines[i])
+            current_code_envir = LANG + codetype + postfix + postfix_err or 'dat'
+            prog = envir2syntax.get(LANG, '')
             lines[i] = ""
         elif lines[i].startswith('!ec'):
-            if current_code_envir is None:
-                # No envir set by previous !bc?
-                errwarn('*** error: mismatch between !bc and !ec')
-                errwarn('    found !ec without a preceding !bc at line')
-                errwarn('\n'.join(lines[i - 8:i - 1]))
-                errwarn('error line >>>', lines[i])
-                errwarn('\n'.join(lines[i + 1:i + 8]))
-                _abort()
             # See if code has to be shown and executed, then format it
             lines[i] = ''
-            formatted_code, comment, execute, show = format_code(current_code,
+            # Determine if the code should be executed, shown, and format it
+            # The globals.py > syntax_executable var lists all supported languages
+            formatted_code, comment, execute, show, postfix_err = format_code(current_code,
                                                                  current_code_envir,
                                                                  code_style,
                                                                  format)
@@ -222,16 +326,39 @@ def process_code_blocks(filestr, code_style, format):
             # Execute and/or show the code and its output
             formatted_output = ''
             if execute:
-                formatted_output, execution_count_ = execute_code_block(
-                    current_code=current_code,
-                    current_code_envir=current_code_envir,
-                    kernel_client=kernel_client,
-                    format=format,
-                    code_style=code_style,
-                    caption=caption,
-                    label=label)
-            if show is not 'hide':
-                if show is not 'output':
+                # Note: the following if-statement is not necessary;
+                # get_execute_show returns execute=False for syntax not in syntax_executable
+                if envir2syntax.get(LANG, '') in syntax_executable:
+                    if prog not in kernel_client:
+                        start_kernel(prog, kernel_client)
+                    if kernel_client[prog].is_alive():
+                        formatted_output, execution_count_, error = execute_code_block(
+                            current_code=current_code,
+                            current_code_envir=current_code_envir, # LANG + codetype + postfix + postfix_err
+                            kernel_client=kernel_client[prog],
+                            format=format,
+                            code_style=code_style,
+                            caption=caption,
+                            label=label)
+                    else:
+                        # This is not used yet because syntax_executable is used in format_code to set execute to False
+                        formatted_output, execution_count, error = execute_subprocess(code=current_code,
+                                           syntax=envir2syntax.get(LANG, ''),
+                                           code_style=code_style,
+                                           format=format)
+                # Warn and abort on code errors
+                if error != '':
+                    if misc.option('execute=') == 'abort':
+                        errwarn('*** error: Error in code block:')
+                    else:
+                        errwarn('*** Warning: found error in code block:')
+                    errwarn('    %s' % error)
+                    errwarn('***')
+                    if misc.option('execute=') == 'abort' and not postfix_err:
+                        _abort()
+            # Format a code cell and its output
+            if show != 'hide':
+                if show != 'output':
                     execution_count += 1
                 formatted_code = format_cell(formatted_code, formatted_output, execution_count, show, format)
                 lines[i] = comment + formatted_code
@@ -243,8 +370,10 @@ def process_code_blocks(filestr, code_style, format):
                 current_code += lines[i] + "\n"
                 lines[i] = ""
 
-    if option("execute"):
-        stop(kernel_client)
+    # Stop all Jupyter kernels
+    #if option("execute"):
+    for kernel in kernel_client.values():
+        kernel.stop()
 
     # Remove empty lines for the html format
     if format in ['html']:
@@ -254,40 +383,73 @@ def process_code_blocks(filestr, code_style, format):
     return filestr
 
 
+def check_errors_in_code_output(outputs, execution_count):
+    """Check if an executed code block resulted in an error
+
+    :param List[str] outputs: execution output
+    :return: error
+    :rtype: str
+    """
+    error = ''
+    # Extract any code error
+    for output in outputs:
+        ## Extract any code error
+        if 'output_type' in output and output['output_type'] == 'error':
+            traceback = ''
+            ansi_escape = re.compile(r'\x1b[^m]*m')
+            if "traceback" in output:
+                traceback = "\n".join(output["traceback"])
+                # Escape characters
+                traceback = ansi_escape.sub("", traceback)
+            error = unescape(traceback) or 'block #%d' % execution_count
+            break
+    return error
+
+
 def execute_code_block(current_code, current_code_envir, kernel_client, format, code_style='', caption=None, label=None):
     """
     Execute a code block and return it formatted together with any code output
 
     Execute a code block in a jupyter kernel, process the code's output and return a formatted string
-    :param str current_code: line of
-    :param str current_code_envir:
+    :param str current_code: code block
+    :param str current_code_envir: LANG + codetype + postfix + postfix_err
     :param JupyterKernelClient kernel_client: instance of JupyterKernelClient
     :param str format: output formatting, one of ['html', 'latex', 'pdflatex']
     :param str code_style: optional typesetting of code blocks
-    :return: text_out, execution_count
-    :rtype: str, int
+    :return: text_out, execution_count, error
+    :rtype: str, int, str
     """
     text_out = ''
     execution_count = 0
-    if kernel_client is None:
+    LANG, codetype, postfix, postfix_err = get_code_block_args('!bc ' + current_code_envir)
+    # Execute the code
+    if kernel_client is None or not kernel_client.is_alive():
         return text_out, execution_count
-    if current_code_envir.endswith('out'):      # Output cell
+    elif postfix == '-out' or postfix == "-noex": # Output cell, or do not execute
+        # Skip executing the code
         outputs = [{'text': current_code}]
     else:
-        outputs, execution_count = run_cell(kernel_client, current_code)
-    if current_code_envir.endswith("hid"):      # Hide cell output
+        outputs, execution_count = kernel_client.run_cell(current_code)
+    # Extract any error in code
+    error = check_errors_in_code_output(outputs, execution_count)
+    # Process the output from execution
+    if postfix == '-hid':      # Hide cell output
         # Skip writing the output, except for ipynb
         if format != 'ipynb':
             outputs = []
-    elif current_code_envir.endswith("-e"):     # Hide also in ipynb
+    elif postfix == '-e':     # Hide also in ipynb
         # Skip writing the output even in ipynb
         outputs = []
     if len(outputs) > 0:
         ansi_escape = re.compile(r'\x1b[^m]*m')
         begin, end = formatted_code_envir("pyout", code_style, format)
         for output in outputs:
+            traceback = ''
             if "text" in output:
-                text_output = ansi_escape.sub("", output["text"])
+                text_output = output["text"]
+                if format in ['html']:
+                    text_output = escape(text_output)
+                text_output = ansi_escape.sub("", text_output)
                 text_out += "\n{}\n{}{}".format(begin, text_output, end)
             if "data" in output:
                 data = output["data"]
@@ -326,10 +488,12 @@ def execute_code_block(current_code, current_code_envir, kernel_client, format,
             elif "traceback" in output:
                 # TODO: convert ANSI escape chars to colors
                 traceback = "\n".join(output["traceback"])
+                # Escape characters
                 traceback = ansi_escape.sub("", traceback)
+                if format in ['html']:
+                    traceback = escape(traceback)
                 text_out += "\n{}\n{}{}".format(begin, traceback, end)
-
-    return text_out, execution_count
+    return text_out, execution_count, error
 
 
 def format_code(code_block, code_block_type, code_style, format):
@@ -342,46 +506,54 @@ def format_code(code_block, code_block_type, code_style, format):
     :param str code_block_type: block type e.g. 'pycod-e'
     :param str code_style: optional typesetting of code blocks
     :param str format: output format, one of ['html', 'latex', 'pdflatex']
-    :return:
+    :return: formatted_code, comment, execute, show (see the invoked function)
+    :rtype: Tuple[str, str, bool, str]
     """
-    postfix = process_code_envir_postfix(code_block_type)
-    execute, show = get_execute_show(postfix)
-    if len(postfix):
-        code_block_type = code_block_type[:-len(postfix)]
-    if format in ['latex','pdflatex']:
+    LANG, codetype, postfix, postfix_err = get_code_block_args('!bc ' + code_block_type)
+    execute, show = get_execute_show((LANG, codetype, postfix))
+    if format in ['latex', 'pdflatex']:
         from .latex import format_code_latex
-        return format_code_latex(code_block, code_block_type, code_style, postfix, execute, show)
+        return format_code_latex(code_block, (LANG, codetype, postfix, postfix_err), code_style, postfix, execute, show)
     elif format in ['html']:
         from .html import format_code_html
-        return format_code_html(code_block, code_block_type, code_style, postfix, execute, show)
+        return format_code_html(code_block, (LANG, codetype, postfix, postfix_err), code_style, postfix, execute, show)
 
 
-def get_execute_show(postfix):
+def get_execute_show(envir):
     """Return whether the code should be executed and showed
 
     Based on the postfix (e.g. '-e') on a code environment (e.g. 'pycod-e'),
-    return whether the code should be executed and shown
-    :param str postfix: postfix (e.g. '-e') to code environment (e.g. 'pycod-e')
+    return whether the code should be executed and shown in the output file.
+    The output `show` is one of ['format','pre','hide','text','output','html'].
+    :param tuple[str, str, str] envir: code blocks arguments e.g. ('py','cod','-h')
     :return: execute, show
     :rtype: bool, str
     """
+    (LANG, codetype, postfix) = envir
+    prog = envir2syntax.get(LANG, '')
     execute = True
     show = 'format'
     if postfix == '-h':     # Show/Hide button (in html)
         execute = False
-    elif postfix == 'hid':  # Hide the cell
+    elif postfix == '-hid': # Hide the cell
         execute = True
         show = 'hide'
     elif postfix == '-e':   # Hide also in ipynb
         execute = True
         show = 'hide'
-    elif postfix == 'out':  # Output cell
+    elif postfix == '-out': # Output cell
         execute = True      # execute_code_block will render this as code output
         show = 'output'
-        return execute, show
     elif postfix == '-t':   # Code as text
         execute = False
         show = 'text'
+    elif postfix == '-noex':   # Do not execute (but show it formatted)
+        execute = False
+    # Only execute python
+    if not option('execute'):
+        execute = False
+    elif prog not in syntax_executable:
+        execute = False
     return execute, show
 
 
@@ -434,15 +606,15 @@ def html_code_envir(envir, envir_spec):
     Return html tags that can be used to wrap formatted code
 
     This method was created to enhance modularization of code. See latex_code_envir in latex.py
-    :param str envir: code environment e.g. "pycod"
+    :param tuple[str, str, str] envir: code blocks arguments e.g. ('py','cod','-h')
     :param str envir_spec: optional typesetting of code blocks
     :return: tuple of html tags, e.g. ('<pre>','</pre>')
     :rtype: [str, str]
     """
     begin, end = '<pre>', '</pre>'
-    if envir_spec.endswith("out") and option("ignore_output"):
+    if envir_spec.endswith('-out') and option('ignore_output'):
         begin, end = '',''
-    elif envir_spec.endswith("-e"):
+    elif envir_spec.endswith('-e'):
         begin, end = '',''
     return begin, end
 
@@ -453,17 +625,19 @@ def latex_code_envir(envir, envir_spec):
 
     Given any code environments and any typesetting of code blocks, return a latex environment
     that can be used to wrap the formatted code
-    :param envir: code environment e.g. "pycod"
+    :param tuple[str, str, str] envir: code blocks arguments e.g. ('py','cod','-h')
     :param envir_spec: optional typesetting of code blocks
     :return: tuple of latex environments, e.g. ('\\bpycod', '\\epycod')
     :rtype: [str, str]
     """
     if envir_spec is None:
-        return '\\b' + envir, '\\e' + envir
-    elif envir.endswith("out") and option("ignore_output"):
+        return '\\b' + envir[0] + envir[1], '\\e' + envir[0] + envir[1]
+    elif envir[2].endswith('-out') and option('ignore_output'):
         return '',''
-    elif envir.endswith("-e"):
+    elif envir[2].endswith('-e'):
         return '',''
+    # Untested old code in the case --latex_code_style is used
+    envir = ''.join(envir)
 
     leftmargin = option('latex_code_leftmargin=', '2')
     bg_vpad = '_vpad' if option('latex_code_bg_vpad') else ''
@@ -506,37 +680,10 @@ def latex_code_envir(envir, envir_spec):
     elif envir.endswith('cod'):
         envir_tp = 'cod'
         envir = envir[:-3]
-
-    # Mappings from DocOnce code environments to Pygments and lstlisting names
-    envir2lst = dict(
-        pyshell='Python',
-        py='Python', cy='Python', f='Fortran',
-        c='C', cpp='C++', bash='bash', sh='bash', rst='text',
-        m='Matlab', pl='Perl', swig='C++',
-        latex='TeX', html='HTML', js='Java',
-        java='Java',
-        xml='XML', rb='Ruby', sys='bash',
-        dat='text', txt='text', csv='text',
-        ipy='Python', do='text',
-        # pyopt and pysc are treated explicitly
-        r='r', php='php',
-    )
-
-    envir2pygments = dict(
-        pyshell='python',
-        py='python', cy='cython', f='fortran',
-        c='c', cpp='c++', cu='cuda', cuda='cuda', sh='bash', rst='rst', swig='c++',
-        bash='bash',
-        m='matlab', pl='perl',
-        latex='latex',
-        html='html',
-        xml='xml', rb='ruby', sys='console',
-        js='js', java='java',
-        dat='text', txt='text', csv='text',
-        ipy='ipy', do='doconce',
-        # pyopt and pysc are treated explicitly
-        r='r', php='php'
-    )
+    # Mappings from DocOnce code environments to lstlisting names
+    # Capitalization does not seem to matter, so I modify envir2syntax
+    envir2lst = envir2syntax
+    envir2lst.update({'rst': 'text', 'latex': 'TeX', 'js': 'Java', 'sys': 'bash', 'ipy': 'python', 'do': 'text'})
 
     if envir in ('ipy', 'do'):
         # Find substitutes for ipy and doconce if these lexers
@@ -547,14 +694,14 @@ def latex_code_envir(envir, envir_spec):
         try:
             get_lexer_by_name('ipy')
         except:
-            envir2pygments['ipy'] = 'python'
+            envir2syntax['ipy'] = 'python'
         try:
             get_lexer_by_name('doconce')
         except:
-            envir2pygments['do'] = 'text'
+            envir2syntax['do'] = 'text'
 
     if package == 'pyg':
-        begin = '\\begin{minted}[%s]{%s}' % (pyg_style, envir2pygments.get(envir, 'text'))
+        begin = '\\begin{minted}[%s]{%s}' % (pyg_style, envir2syntax.get(envir, 'text'))
         end = '\\end{minted}'
     elif package == 'lst':
         if envir2lst.get(envir, 'text') == 'text':
