@@ -1,18 +1,20 @@
-import os, sys, shutil, re, glob, math
+import os, sys, shutil, glob, math
+import regex as re
 from doconce import globals
 from .doconce import read_file, write_file, doconce2format, handle_index_and_bib, preprocess
-from .misc import option, help_print_options, check_command_line_options, system, _abort, find_file_with_extensions, \
-    _rmdolog, errwarn,  debugpr
+from .misc import option, help_print_options, check_command_line_options, system, _abort, \
+    find_file_with_extensions, folder_checker, doconce_version, _rmdolog, errwarn, debugpr
 from .common import INLINE_TAGS, remove_code_and_tex
 import json
 from .ipynb import img2ipynb
 from .html import movie2html
 
-docstring_jupyterbook = """Usage: doconce jupyterbook filename [options] 
-Create files for Jupyter Book version: 0.8
-Example: doconce jupyterbook filename.do.txt --sep=chapter --sep_section=subsection --show_titles
-Try 'doconce jupyterbook --help' for more information.
-"""
+docstring_jupyterbook = ('Usage:\n'
+                         '\033[1mdoconce jupyterbook <file>[.do.txt] [options]\033[0m\n'
+                         'Create directories and files for Jupyter Book version: 0.8\n'
+                         '\n'
+                         'Example:\n'
+                         'doconce jupyterbook filename.do.txt --sep=chapter --sep_section=subsection --show_titles\n')
 
 _registered_cmdline_opts_jupyterbook = [
     ('-h', 'Show this help page'),
@@ -43,7 +45,9 @@ def jupyterbook():
     """
     # Print help
     if len(sys.argv) < 2:
+        doconce_version()
         print(docstring_jupyterbook)
+        print("Try 'doconce jupyterbook --help' for more information.")
         sys.exit(1)
     if option('help') or '-h' in sys.argv:
         print_help_jupyterbook()
@@ -71,9 +75,19 @@ def jupyterbook():
     if not filename:
         errwarn('*** error: file %s does not exist' % globals.filename)
         _abort()
+    globals.dirname = dirname
     if dirname:
+        # cd into the DocOnce file's directory, then fix dest and dest_toc
         os.chdir(dirname)
         errwarn('*** doconce format now works in directory %s' % dirname)
+        # fix dest, dest_roc, and finally dirname
+        dest = os.path.relpath(dest or '.', start=dirname) + '/'
+        if dest.startswith('./'):
+            dest = dest[2:]
+        dest_toc = os.path.relpath(dest_toc or '.', start=dirname) + '/'
+        if dest_toc.startswith('./'):
+            dest_toc = dest_toc[2:]
+        #dirname = ''
     globals.filename = filename
     globals.dofile_basename = basename
 
@@ -254,7 +268,7 @@ def jupyterbook():
     all_texts_formatted = resolve_links_destinations(all_texts_formatted, all_basenames)
     # Fix the path of FIGUREs and MOVIEs.
     # NB: at the time of writing (03-2021) movies are not supported by Jupyter Book
-    all_texts_formatted = [fix_media_src(t, dirname, dest) for t in all_texts_formatted]
+    all_texts_formatted = [fix_media_src(t, '', dest) for t in all_texts_formatted]
     # Write chapters and sections to file
     for i in range(len(all_texts_formatted)):
         write_file(all_texts_formatted[i], dest + all_fnames[i], _encoding=globals.encoding)
@@ -327,14 +341,14 @@ def split_ipynb(ipynb_text, filenames):
     if len(ind_fname) != len(filenames):
         errwarn('*** error : could not find all markings in ipynb')
         _abort()
-    # Create an ipynb dictionary for each block, then convert to text
+    # For each file create a dictionary with the relevant ipynb blocks, then convert to text
     ipynb_texts = [''] * len(filenames)
-    for i, ind in enumerate(ind_fname):
-        ind2 = None
-        if ind + 1 < len(ind_fname):
-            ind2 = ind_fname[ind + 1]
+    for i, ind_start in enumerate(ind_fname):
+        ind_end = None
+        if i + 1 < len(ind_fname):
+            ind_end = ind_fname[i + 1]
         block_dict = ipynb_dict.copy()
-        block_dict['cells'] = cells[ind:ind2]
+        block_dict['cells'] = cells[ind_start:ind_end]
         ipynb_texts[i] = json.dumps(block_dict, indent=1, separators=(',', ':'))
     return ipynb_texts
 
@@ -352,7 +366,7 @@ def read_title_file(titles_opt, chapters, sec_list):
     """
     chapter_titles = []
     sec_title_list = [[]] * len(chapters)
-    if titles_opt is not 'auto':
+    if titles_opt != 'auto':
         chapter_titles = [''] * len(chapters)
         input_titles = read_to_list(titles_opt)
         for c in range(len(chapters)):
@@ -489,7 +503,7 @@ def create_toc_yml(basenames, nesting_levels, titles, dest='./', dest_toc='./',
             title = '"' + title + '"'
         return title
     # Get the relative path between the destination folders
-    relpath = os.path.relpath(dest, dest_toc)
+    relpath = os.path.relpath(dest, start=dest_toc)
     if relpath == '.':
         relpath = ''
     else:
@@ -519,8 +533,8 @@ def print_help_jupyterbook():
 
     Help function to print help and formatted command line options for doconce jupyterbook
     """
-    from .misc import help_format
     print(docstring_jupyterbook)
+    print('Options:')
     help_print_options(cmdline_opts=_registered_cmdline_opts_jupyterbook)
 
 
@@ -542,23 +556,6 @@ def read_to_list(file):
     return out
 
 
-def folder_checker(dirname):
-    """Verify a path
-
-    Helper function to evaluate the existence and formatting of a folder
-
-    :param str dirname: Path to an existing folder
-    :return: directory name
-    :rtype: str
-    """
-    if not os.path.isdir(dirname):
-        errwarn('*** error : destination folder not found:\n    %s' % dirname)
-        _abort()
-    if not dirname.endswith('/'):
-        return dirname + '/'
-    return dirname
-
-
 def get_link_destinations(chunk):
     """Find any target of a link in HTML code
 
@@ -570,7 +567,10 @@ def get_link_destinations(chunk):
     destinations, destination_tags = [], []
     # html links. label{} has already been converted
     pattern_tag = r'[\w _\-:]'
-    pattern = r'<' + pattern_tag + '+ (id|name)=["\'](' + pattern_tag + '+)["\'][^>]*>'
+    pattern_backslash = '[\\\]'
+    pattern = r'<' + pattern_tag + \
+              '+ (id|name)=' + pattern_backslash + '["\']' + \
+              '(' + pattern_tag + '+)' + pattern_backslash + '["\'][^>]*>'
     for m in re.finditer(pattern, chunk):
         match = m.group()
         tag = m.group(2)
@@ -646,9 +646,11 @@ def fix_media_src(filestr, dirname, dest):
     The generated .md and .ipynb files will be created in the path passed to `--dest`.
     This method fixes the paths of the image and movie files so that they can be found
     in generated .md and .ipynb files.
-    :param filestr:
-    :param dirname:
-    :return:
+    :param str filestr: text string
+    :param str dirname: Path to an existing folder
+    :param str dest: directory name
+    :return: filestr with new paths
+    :rtype: str
     """
     patterns = [
         # movies in .md and .ipynb. NB: jupyterbook does not support movies
@@ -677,10 +679,11 @@ def fix_media_src(filestr, dirname, dest):
                          '  myst_enable_extensions:\n'
                          '    - html_image\n'))
             if not src.startswith('/'):
-                src_new = os.path.relpath(dirname + src, dest)
+                if dirname != '' and not dirname.endswith('/'):
+                    dirname += '/'
+                src_new = os.path.relpath(dirname + src, start=dest)
                 replacement = match.replace(src, src_new, 1)
                 filestr_out = filestr_out.replace(match, replacement, 1)
-                #print('\npattern #%d replaced \n   %s  \nwith\n   %s\n' % (i, match, replacement))
     return filestr_out
 
 
