@@ -35,11 +35,13 @@ from future import standard_library
 standard_library.install_aliases()
 from builtins import range
 
-import re, os, subprocess, sys, subprocess
-from .common import default_movie, plain_exercise, insert_code_blocks, insert_tex_blocks
+import os, subprocess, sys, subprocess
+import regex as re
+import shlex
+from .common import default_movie, plain_exercise, insert_code_blocks, insert_tex_blocks, table_analysis
 from .plaintext import plain_quiz
 from .misc import _abort, errwarn
-from .globals import postfix_regex
+from .globals import postfix_regex, code_block_regex
 
 
 def align2equations(math_text):
@@ -53,15 +55,13 @@ def align2equations(math_text):
     if not '{align' in math_text:
         return
     math_text = math_text.replace('&', '')
-    math_text = math_text.replace('\\\\', r"""
-</math>
-
-:<math>""")
+    math_text = math_text.replace('\\\\', ('\n</math>\n\n:<math>\n'))
     pattern = r'\\(begin|end)\{align\*?\}\s*'
     math_text = re.sub(pattern, '', math_text)
     # :<math> and </math> surroundings appear when !bt and !et are translated
     return math_text
 
+
 def equation2nothing(math_text):
     pattern = r'\\(begin|end)\{equation\*?\}\s*'
     math_text = re.sub(pattern, '', math_text)
@@ -69,6 +69,7 @@ def equation2nothing(math_text):
     math_text = math_text.replace(r'\]', '')
     return math_text
 
+
 def remove_labels(math_text):
     pattern = 'label\{(.+?)\}\s*'
     labels = re.findall(pattern, math_text)
@@ -110,13 +111,12 @@ def mwiki_code(filestr, code_blocks, code_block_types,
                       javacod='java', javapro='java',
                       htmlcod='html5', xmlcod='xml',
                       htmlpro='html5', xmlpro='xml',
-                      html='html5', xml='xml',
-                      sys='bash', dat='text', csv='text', txt='text',
+                      html='html5', xml='xml', sys='bash',
+                      dat='text', txt='text', csv='text',
                       pyoptpro='python', pyscpro='python',
                       ipy='python', pyshell='python',
                       )
 
-    code_block_regex = r'^!bc\s+%s' + postfix_regex + '\s*\n'
     for key in envir2lang:
         language = envir2lang[key]
         cpattern = re.compile(code_block_regex % key, flags=re.MULTILINE)
@@ -139,7 +139,20 @@ def mwiki_code(filestr, code_blocks, code_block_types,
 
 
 def mwiki_figure(m):
-    filename = m.group('filename')
+    """Format figures to the mwiki format
+
+    Return mwiki code to embed a figure in the .mwiki output. The syntax is
+    `FIGURE:[filename[, options][, sidecap=BOOL][, frac=NUM]] [caption]`.
+    Keywords: `sidecap` (default is False), `frac` (default is ),
+    :param _regex.Match m: regex match object
+    :return: mwiki code
+    :rtype: str
+    """
+    caption = m.group('caption').strip().strip('"').strip("'")
+    filename = m.group('filename').strip()
+    opts = m.group('options').strip()
+    info = dict()
+
     link = filename if filename.startswith('http') else None
     if not link and not os.path.isfile(filename):
         raise IOError('no figure file %s' % filename)
@@ -161,7 +174,6 @@ def mwiki_figure(m):
                 _abort()
             filename = root + '.png'
 
-    caption = m.group('caption').strip()
     if caption != '':
         caption = '|' + caption  # add | for non-empty caption
     else:
@@ -172,23 +184,23 @@ def mwiki_figure(m):
     caption = re.sub(r'label\{(.+?)\}', '(\g<1>)', caption)
 
     size = ''
-    opts = m.group('options').strip()
     if opts:
-        info = dict([s.split('=') for s in opts.split()])
+        info = shlex.split(opts)
+        info = dict(s.strip(',').split('=') for s in info)
         if 'width' in info and 'height' in info:
-            size = '|%sx%spx' % (info['width'], info['height'])
+            size = '|%sx%spx' % (info['width'].replace('px',''), info['height'].replace('px',''))
         elif 'width' in info:
-            size = '|%spx' % info['width']
+            size = '|%spx' % info['width'].replace('px','')
         elif 'height' in info:
-            size = '|x%spx' % info['height']
+            size = '|x%spx' % info['height'].replace('px','')
 
     if link:
         # We link to some image on the web
         filename = os.path.basename(filename)
         link = os.path.dirname(link)
-        result = r"""
-[[File:%s|frame%s|link=%s|alt=%s%s]]
-""" % (filename, size, link, filename, caption)
+        result = ('\n'
+                  r'[[File:%s|frame%s|link=%s|alt=%s%s]]'
+                  '\n\n') % (filename, size, link, filename, caption)
     else:
         # We try to link to a file at wikimedia.org.
         found_wikimedia = False
@@ -233,9 +245,9 @@ def mwiki_figure(m):
             if user:
                 found_wikimedia = True
                 errwarn(' ...found %s at wikimedia' % filename)
-                result = r"""
-    [[File:%s|frame%s|alt=%s%s]] <!-- user: %s, filename: %s, timestamp: %s -->
-    """ % (filename, size, filename, caption, user, orig_filename, timestamp)
+                result = ('\n'
+                          r'    [[File:%s|frame%s|alt=%s%s]] <!-- user: %s, filename: %s, timestamp: %s -->'
+                          '    \n') % (filename, size, filename, caption, user, orig_filename, timestamp)
         except IOError:
             errwarn(' ...no Internet connection...')
 
@@ -246,14 +258,12 @@ def mwiki_figure(m):
             errwarn(' ...for now we use local file %s' % filename)
             # This is fine if we use github wiki
 
-            result = r"""
-[[File:%s|frame%s|alt=%s%s]] <!-- not yet uploaded to common.wikimedia.org -->
-""" % (filename, size, filename, caption)
+            result = ('\n'
+                      r'[[File:%s|frame%s|alt=%s%s]] <!-- not yet uploaded to common.wikimedia.org -->'
+                      '\n\n') % (filename, size, filename, caption)
 
     return result
 
-from .common import table_analysis
-
 
 def mwiki_author(authors_and_institutions, auth2index,
                  inst2index, index2inst, auth2email):
@@ -283,9 +293,11 @@ def mwiki_author(authors_and_institutions, auth2index,
 
 from .gwiki import wiki_ref_and_label_common
 
+
 def mwiki_ref_and_label(section_label2title, format, filestr):
     return wiki_ref_and_label_common(section_label2title, format, filestr)
 
+
 def mwiki_admon(block, format, title='Warning', text_size='normal',
                 admon_type='warning'):
     if title.lower().strip() == 'none':
@@ -340,6 +352,7 @@ def mwiki_admon(block, format, title='Warning', text_size='normal',
 
 # mbox: notice
 
+
 def define(FILENAME_EXTENSION,
            BLANKLINE,
            INLINE_TAGS_SUBST,
