@@ -12,12 +12,14 @@ from past.builtins import basestring
 # solution seems to be manual editing, which is doable for small
 # documents and issues.)
 
-import re, sys, functools, string
+import sys, functools, string
+import regex as re
+import shlex
 from .common import default_movie, plain_exercise, table_analysis, DEFAULT_ARGLIST, \
      insert_code_blocks, insert_tex_blocks, bibliography, indent_lines, fix_ref_section_chapter
 from .html import html_movie, html_table
 from .misc import option, errwarn
-from .globals import postfix_regex
+from .globals import postfix_regex, code_block_regex
 
 # Mapping of envirs to correct Pandoc verbatim environment
 language2pandoc = dict(
@@ -172,7 +174,6 @@ def pandoc_code(filestr, code_blocks, code_block_types,
 
         # Code blocks apply the ~~~~~ delimiter, with blank lines before
         # and after
-        code_block_regex = r'^!bc\s+%s' + postfix_regex + '\s*\n'
         for key in language2pandoc:
             language = language2pandoc[key]
             if github_md:
@@ -279,14 +280,27 @@ def pandoc_table(table):
     return s
 
 def pandoc_figure(m):
-    filename = m.group('filename')
-    caption = m.group('caption').strip()
-    opts = m.group('options').strip()
+    """Format figures to the pandoc format
 
+    Return pandoc code to embed a figure in the sphinx .md output. The syntax is
+    `FIGURE:[filename[, options][, sidecap=BOOL][, frac=NUM]] [caption]`.
+    Keywords: `sidecap` (default is False), `frac` (default is ),
+    :param _regex.Match m: regex match object
+    :return: pandoc code
+    :rtype: str
+    """
+    filename = m.group('filename').strip()
+    caption = m.group('caption').strip().strip('"').strip("'")
+    opts = m.group('options').strip()
+    info = dict()
+
+    # Process any inline figure opts
     if opts:
-        info = [s.split('=') for s in opts.split()]
-        opts = ' '.join(['%s=%s' % (opt, value)
-                         for opt, value in info
+        info = shlex.split(opts)
+        info = dict(s.strip(',').split('=') for s in info)
+        # String of options
+        opts = ' '.join(['%s="%s"' % (opt, value)
+                         for opt, value in info.items()
                          if opt not in ['frac', 'sidecap']])
 
     # Save raw html with width etc in a comment so we have that info
